/*************************************************************************
|  COPYRIGHT (c) 2000 BY ABATRON AG
|*************************************************************************
|
|  PROJECT NAME: Simple bdiAccess demo
|  FILENAME    : myapp.c
|
|  COMPILER    : GCC
|
|  TARGET OS   : Linux
|  TARGET HW   : PC
|
|  PROGRAMMER  : Abatron / RD
|  CREATION    : 14.07.00
|
|*************************************************************************
|
|  DESCRIPTION :
|  A simple command line application that used bdiAccess functions.
|
|
|*************************************************************************/

/*************************************************************************
|  INCLUDES
|*************************************************************************/

#include <ctype.h>
#include <unistd.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/times.h>

#include "bdierror.h"
#include "bdicmd.h"
#include "bdilnk.h"
#include "bdiifc.h"


/*************************************************************************
|  DEFINES
|*************************************************************************/

#define TARGET_OKAY   0


/*************************************************************************
|  MACROS
|*************************************************************************/

/*************************************************************************
|  TYPEDEFS
|*************************************************************************/

/*************************************************************************
|  LOCALS
|*************************************************************************/

static DWORD      nextTargetAddr;  /* used for continous memory display */
static BOOL       terminate;
static BOOL       modeJTAG;


/****************************************************************************
 ****************************************************************************

    ValidEndOfLine:

    Test for a valid enf of line

    INPUT  : linePtr    pointer to the next character to read
    OUTPUT : RETURN     TRUE if valid end of line

 ****************************************************************************/

static BOOL ValidEndOfLine (const char* linePtr)
{
  while (*linePtr == ' ') linePtr++;
  return (*linePtr == 0);
} /* ValidEndOfLine */


/****************************************************************************
 ****************************************************************************

    ExtractNumber

    Extract a number (0xnnn for hex). Conversion stops if illegal character.

    INPUT  : getPtr     pointer to the next character to read
    OUTPUT : value      the extracted integer
             RETURN     pointer to the next unread character

 ****************************************************************************/

static char* ExtractHex (DWORD* value,  char* getPtr)
{
  DWORD num;
  DWORD digit;
  BYTE  c;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (;;) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (DWORD)(c - '0');
    else if ((c >= 'A') && (c <= 'F')) digit = (DWORD)(c - 'A' + 10);
    else if ((c >= 'a') && (c <= 'f')) digit = (DWORD)(c - 'a' + 10);
    else break;
    num <<= 4;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractHex */

static char* ExtractDecimal (DWORD* value,  char* getPtr)
{
  DWORD num;
  DWORD digit;
  BYTE  c;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (;;) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (DWORD)(c - '0');
    else break;
    num *= 10;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractDecimal */


static char* ExtractNumber (DWORD* value,  char* getPtr)
{
  while (*getPtr == ' ') getPtr++;
  if ((*getPtr == '0') && ((*(getPtr+1) == 'x') || (*(getPtr+1) == 'X'))) {
    getPtr +=2;
    return ExtractHex(value, getPtr);
  } /* if */
  else {
    return ExtractDecimal(value, getPtr);
  } /* else */
} /* ExtractNumber */


/****************************************************************************
 ****************************************************************************

    MatchName

    Searches for a name in a name table. The search is not case sensitive.
    All transmitted characters until the next special character are compared.
    At least one name character must be present.

    INPUT  : tableSize  the number of enties in the name table
             nameTable  the name table to search
             getPtr     pointer to the command frame
    OUTPUT : index      the index if a match is found or -1 if no match
             RETURN     pointer to the next unread character in the command frame

 ****************************************************************************/

static char* MatchName(int tableSize, char** nameTable, int* index, char* getPtr)
{
  int   i;
  char  firstChar;
  char  c;
  char* name;
  char* reference;

  firstChar = *getPtr;
  if (firstChar >= 'a') firstChar -= ('a'-'A'); /* make upper */
  for (i=0; i < tableSize; i++) {
    reference = *nameTable;
    if (*reference++ == firstChar) {
      name = getPtr;
      do {
        name++;
        c = *name;

        /* valid end */
        if (c <= ' ') {
          *index = i;
          return name;
        } /* if */

        if (c >= 'a') c -= ('a'-'A'); /* make upper */

        /* check invalid char */
        if (!(c >= 'A' && c <= 'Z') && !(c >= '0' && c <= '9')) {
          *index = -1;
          return name;
        } /* if */

      } while (*reference++ == c);

    } /* if */
    nameTable++;
  } /* for */

  /* no match found */
  *index = -1;
  return getPtr;
} /* MatchName */


/****************************************************************************
 ****************************************************************************

    DisplayHelp :

    Display the help screen.

    INPUT  : -
    OUTPUT : -

 ****************************************************************************/

static char* helpScreen[] = {
"MD    [<address>] [<count>]  display target memory as word (32bit)",
"MDH   [<address>] [<count>]  display target memory as half word (16bit)",
"MDB   [<address>] [<count>]  display target memory as byte (8bit)",
"MM    <address>   <value>    modify word (32bit) to target memory",
"MMH   <address>   <value>    modify half word (16bit) to target memory",
"MMB   <address>   <value>    modify byte (8bit) to target memory",
"STARTUP                      reset and init target",
"INFO                         display information about the target",
"HELP                         display command list",
"EXIT                         terminate"
};

static void DisplayHelp(void)
{
  int   i;
  char  line[80];

  for (i=0; i < (int)(sizeof helpScreen / sizeof helpScreen[0]); i++) {
    printf("%s\n", helpScreen[i]);
    if ((i % 20) == 19) {
      printf("press enter to continue...");
      gets(line);
    } /* if */
  } /* for */
} /* DisplayHelp */


/****************************************************************************
 ****************************************************************************

    DisplayLong :

    Display memory words (32bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayLong(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  DWORD mem[4];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<4; j++) {
        if (result == TARGET_OKAY) result = BDI_GetLong(address + 4 * j,  &mem[j]);
      } /* for */
      if (result == TARGET_OKAY) {
        (void)printf("%08lx : %08lx %08lx %08lx %08lx\n", address, mem[0], mem[1], mem[2], mem[3]);
        address += 16;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_GetLong(address, &mem[0]);
    if (result == TARGET_OKAY) {
      (void)printf("%08lx : 0x%08lx  % 11li\n", address, mem[0], mem[0]);
      address += 4;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayLong */


/****************************************************************************
 ****************************************************************************

    DisplayWord :

    Display memory half words (16bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayWord(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  WORD  mem[8];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<8; j++) {
        if (result == TARGET_OKAY) result = BDI_GetWord(address + 2 * j,  &mem[j]);
      } /* for */
      if (result == TARGET_OKAY) {
        printf("%08lx : %04x %04x %04x %04x %04x %04x %04x %04x\n",
               address, mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6],mem[7]);
        address += 16;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requuested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_GetWord(address, &mem[0]);
    if (result == TARGET_OKAY) {
      printf("%08lx : 0x%04x  % 6i\n", address, mem[0], mem[0]);
      address += 2;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayWord */


/****************************************************************************
 ****************************************************************************

    DisplayByte :

    Display memory as bytes (8bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayByte(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  BYTE  mem[8];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<8; j++) {
        if (result == TARGET_OKAY) result = BDI_GetByte(address + j,  &mem[j]);
      } /* for */
      if (result == TARGET_OKAY) {
        printf("%08lx : %02x %02x %02x %02x %02x %02x %02x %02x\n",
               address, mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6],mem[7]);
        address += 8;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requuested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_GetByte(address, &mem[0]);
    if (result == TARGET_OKAY) {
      printf("%08lx : 0x%02x  % 4i\n", address, mem[0], mem[0]);
      address++;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayByte */


/****************************************************************************
 ****************************************************************************

    DisplayTargetInfo :

    Display the target information.

    INPUT  : -
    OUTPUT : -

 ****************************************************************************/

static int DisplayTargetInfo(void)
{
  int       result;
  WORD      state;
  DWORD     timeStamp;
  WORD      infoCount;
  DWORD     infoValue[10];
  int       errorCode;
  WORD      i;

  infoCount = 10;
  result = BDI_StateGet(&state, &errorCode, &timeStamp, &infoCount, infoValue);
  if (result != TARGET_OKAY) return result;

  switch (state) {

    case 0:
      printf("BDI state         : reset\n");
      break;

    case 1:
      printf("Target state      : debug mode\n");
      break;

    case 2:
      printf("BDI state         : running\n");
      break;

    case 3:
      printf("BDI state         : error\n");
      printf("Error info        : %s\n", BDI_ErrorTextGet(errorCode));
      break;

    default:
      printf("BDI state         : unknown\n");
      break;

  } /* switch */

  printf("Reset Timestamp   : %lu\n", timeStamp);

  for (i = 0; i < infoCount; i++) {
    printf("Info[%i]           : 0x%08lx\n", i+1, infoValue[i]);
  } /* while */

  return TARGET_OKAY;
} /* DisplayTargetInfo */


/****************************************************************************
 ****************************************************************************

  TARGET_ReadGPR:
  TARGET_ReadPSR:

  Read a Register.

  INPUT  : regNbr               Number of register
  OUTPUT : value                data
           RETURN               error code

 ****************************************************************************/

static int TARGET_ReadGPR(WORD regNbr, DWORD* value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_GPR;
  reg.regAddr = regNbr;
  return BDI_RegisterGet(1, &reg, value);
} /* TARGET_ReadGPR */

static int TARGET_ReadCPSR(DWORD* value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_CPSR;
  reg.regAddr = 0;
  return BDI_RegisterGet(1, &reg, value);
} /* TARGET_ReadCPSR */


/****************************************************************************
 ****************************************************************************

  TARGET_WriteGPR:
  TARGET_WriteICEBreaker:
  TARGET_WriteCP15:

  Write to Register.

  INPUT  : regNbr       register number
           value        the value to write
  OUTPUT : RETURN       error code

 ****************************************************************************/

static int TARGET_WriteGPR(WORD regNbr, DWORD value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_GPR;
  reg.regAddr = regNbr;
  return BDI_RegisterSet(1, &reg, &value);
} /* TARGET_WriteGPR */

int TARGET_SetTargetPC(DWORD value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_GPR;
  reg.regAddr = 15;
  return BDI_RegisterSet(1, &reg, &value);
} /* TARGET_SetTargetPC */


static int TARGET_WriteICEBreaker (WORD regNbr, DWORD value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_ICEB;
  reg.regAddr = regNbr;
  return BDI_RegisterSet(1, &reg, &value);
} /* TARGET_WriteICEBreaker */

static int TARGET_WriteCP15(WORD regNbr, DWORD value)
{
  BDI_RegTypeT    reg;

  reg.regType = BDI_RT_ARM_CP15;
  reg.regAddr = regNbr;
  return BDI_RegisterSet(1, &reg, &value);
} /* TARGET_WriteCP15 */


/****************************************************************************
 ****************************************************************************

    TELNET_DisplayGPR :

    Display general purpose registers.

    INPUT  : -
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int TELNET_DisplayGPR(void)
{
  int   i;
  int   j;
  int   result;
  DWORD gpr[4];

  static BDI_RegTypeT regList[] = {BDI_RT_ARM_CPSR, 0};

  /* display gereral purpose registers */
  result = TARGET_OKAY;
  for (i=0; i<4; i++) {
      for (j=0; j<4; j++) {
        if (result == TARGET_OKAY) result = TARGET_ReadGPR((WORD)((4 * i) + j), &gpr[j]);
      } /* for */
    if (result == TARGET_OKAY) {
      printf("GPR%02i: %08lx %08lx %08lx %08lx\n", (4*i), gpr[0], gpr[1], gpr[2], gpr[3]);
    } /* if */
    else {
      break;
    } /* else */
  } /* for */

  /* display PSR and PC */
  if (result == TARGET_OKAY) result = BDI_RegisterGet(1, regList, gpr);
  if (result == TARGET_OKAY) {
    printf("CPSR : %08lx\n", gpr[0]);
  } /* if */

  return result;
} /* TELNET_DisplayGPR */


/****************************************************************************
 ****************************************************************************

    TELNET_DisplayICEBreaker :

    Display OnCE registers.

    INPUT  : -
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int TELNET_DisplayICEBreaker(void)
{
  int   result;
  char  line[80];
  DWORD reg[17];

  static BDI_RegTypeT regList[] =
  {
    { BDI_RT_ARM_ICEB,  0 },   // Debug Control
    { BDI_RT_ARM_ICEB,  1 },   // Debug Status
    { BDI_RT_ARM_ICEB,  4 },   // Debug Comms Control
    { BDI_RT_ARM_ICEB,  5 },   // Debug Comms Data
    { BDI_RT_ARM_ICEB,  8 },   // Watchpoint 0 Address Value
    { BDI_RT_ARM_ICEB,  9 },   // Watchpoint 0 Address Mask
    { BDI_RT_ARM_ICEB, 10 },   // Watchpoint 0 Data Value
    { BDI_RT_ARM_ICEB, 11 },   // Watchpoint 0 Data Mask
    { BDI_RT_ARM_ICEB, 12 },   // Watchpoint 0 Control Value
    { BDI_RT_ARM_ICEB, 13 },   // Watchpoint 0 Control Mask
    { BDI_RT_ARM_ICEB, 16 },   // Watchpoint 1 Address Value
    { BDI_RT_ARM_ICEB, 17 },   // Watchpoint 1 Address Mask
    { BDI_RT_ARM_ICEB, 18 },   // Watchpoint 1 Data Value
    { BDI_RT_ARM_ICEB, 19 },   // Watchpoint 1 Data Mask
    { BDI_RT_ARM_ICEB, 20 },   // Watchpoint 1 Control Value
    { BDI_RT_ARM_ICEB, 21 },   // Watchpoint 1 Control Mask
    { BDI_RT_ARM_ICEB,  2 }    // Vector Catch
  };

  result = BDI_RegisterGet(17 , regList, reg);
  if (result == TARGET_OKAY) {
    printf("DBGC: %08lx     DBGS: %08lx\n", reg[0], reg[1]);
    printf("COMC: %08lx     COMD: %08lx\n", reg[2], reg[3]);
    printf("W0AV: %08lx     W0AM: %08lx\n", reg[4], reg[5]);
    printf("W0DV: %08lx     W0DM: %08lx\n", reg[6], reg[7]);
    printf("W0CV: %08lx     W0CM: %08lx\n", reg[8], reg[9]);
    printf("W1AV: %08lx     W1AM: %08lx\n", reg[10], reg[11]);
    printf("W1DV: %08lx     W1DM: %08lx\n", reg[12], reg[13]);
    printf("W1CV: %08lx     W1CM: %08lx\n", reg[14], reg[15]);
    printf("VCATCH: %08lx\n", reg[16]);

  } /* if */

  return result;
} /* TELNET_DisplayICEBreaker */


/****************************************************************************
 ****************************************************************************

    TELNET_DisplayCP15 :

    Display general purpose registers.

    INPUT  :
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int TELNET_DisplayCP15(WORD regNbr)
{
  int           result;
  char          line[80];
  DWORD         value;
  BDI_RegTypeT  reg;

  reg.regType = BDI_RT_ARM_CP15;
  reg.regAddr = regNbr;
  result = BDI_RegisterGet(1, &reg, &value);
  if (result == TARGET_OKAY) {
    printf("CP15 : %08lx\n", value);
  } /* if */

  return result;
} /* TELNET_DisplayCP15 */



/****************************************************************************
 ****************************************************************************

    ProcessCommand :

    Process a command line

    INPUT  : line       the command line
    OUTPUT : -

 ****************************************************************************/


/* Init list for SMDK2410 board */
#define CLOCK_SMDK2410    0x00000601
static const BDI_InitTypeT initListSMDK2410[] = {
  {BDI_INIT_ARM_WGPR, 0x8009,     0x1       }, // Special BDI register: configure TRST as push-pull
  {BDI_INIT_ARM_WM32, 0x15300000, 0x00008000}, // WTCON
  {BDI_INIT_ARM_WM32, 0x14000000, 0x22111114}, // BWSCON
  {BDI_INIT_ARM_WM32, 0x14000004, 0x00000700}, // BANKCON0
  {BDI_INIT_ARM_WM32, 0x14000008, 0x00000700}, // BANKCON1
  {BDI_INIT_ARM_WM32, 0x1400000c, 0x00000700}, // BANKCON2
  {BDI_INIT_ARM_WM32, 0x14000010, 0x00000700}, // BANKCON3
  {BDI_INIT_ARM_WM32, 0x14000014, 0x00000700}, // BANKCON4
  {BDI_INIT_ARM_WM32, 0x14000018, 0x00000700}, // BANKCON5
  {BDI_INIT_ARM_WM32, 0x1400001c, 0x00018005}, // BANKCON6
  {BDI_INIT_ARM_WM32, 0x14000020, 0x00018005}, // BANKCON7
  {BDI_INIT_ARM_WM32, 0x14000024, 0x008e0459}, // REFRESH
  {BDI_INIT_ARM_WM32, 0x14000028, 0x00000032}, // BANKSIZE
  {BDI_INIT_ARM_WM32, 0x1400002c, 0x00000030}, // MRSRB6
  {BDI_INIT_ARM_WM32, 0x14000030, 0x00000030}  // MRSRB7
};


/* Init list for SMDK2400 board */
#define CLOCK_SMDK2400    0x00000601
static const BDI_InitTypeT initListSMDK2400[] = {
  {BDI_INIT_ARM_WGPR, 8009,       1         }, // Special BDI register: configure TRST as push-pull
  {BDI_INIT_ARM_WM32, 0x15300000, 0x00008000}, // WTCON
  {BDI_INIT_ARM_WM32, 0x14000000, 0x22111114}, // BWSCON
  {BDI_INIT_ARM_WM32, 0x14000004, 0x00000700}, // BANKCON0
  {BDI_INIT_ARM_WM32, 0x14000008, 0x00000700}, // BANKCON1
  {BDI_INIT_ARM_WM32, 0x1400000c, 0x00000700}, // BANKCON2
  {BDI_INIT_ARM_WM32, 0x14000010, 0x00000700}, // BANKCON3
  {BDI_INIT_ARM_WM32, 0x14000014, 0x00000700}, // BANKCON4
  {BDI_INIT_ARM_WM32, 0x14000018, 0x00000700}, // BANKCON5
  {BDI_INIT_ARM_WM32, 0x1400001c, 0x00018005}, // BANKCON6
  {BDI_INIT_ARM_WM32, 0x14000020, 0x00018005}, // BANKCON7
  {BDI_INIT_ARM_WM32, 0x14000024, 0x008e0459}, // REFRESH
  {BDI_INIT_ARM_WM32, 0x14000028, 0x00000010}, // BANKSIZE
  {BDI_INIT_ARM_WM32, 0x1400002c, 0x00000030}, // MRSRB6
  {BDI_INIT_ARM_WM32, 0x14000030, 0x00000030}  // MRSRB7
};


/* Init list for AT91RM9200DK board */
#define CLOCK_RM9200    0x00000601
static const BDI_InitTypeT initListRM9200[] = {
  {BDI_INIT_ARM_WCPSR, 0,          0x000000D3}, // select supervisor mode
  {BDI_INIT_ARM_WM32,  0xFFFFFF00, 0x00000001}, // Cancel reset remapping
  {BDI_INIT_ARM_WM32,  0xFFFFFC20, 0x0000FF01}, // PMC_MOR : Enable main oscillator , OSCOUNT = 0xFF
  {BDI_INIT_ARM_WM32,  0xFFFFFF10, 0x00000000}, // MC_PUIA[0]
  {BDI_INIT_ARM_WM32,  0xFFFFFF50, 0x00000000}, // MC_PUP
  {BDI_INIT_ARM_WM32,  0xFFFFFF54, 0x00000000}, // MC_PUER: Memory controller protection unit disable
  {BDI_INIT_ARM_WM32,  0xFFFFFF64, 0x00000000}, // EBI_CFGR
  {BDI_INIT_ARM_WM32,  0xFFFFFF70, 0x00003284}, // SMC2_CSR[0]: 16bit, 2 TDF, 4 WS
  {BDI_INIT_ARM_WM32,  0xFFFFFC28, 0x20263E04}, // PLLAR: 179,712000 MHz for PCK
  {BDI_INIT_ARM_DELAY, 0,	   100       }, //
  {BDI_INIT_ARM_WM32,  0xFFFFFC2C, 0x10483E0E}, // PLLBR: 48,054857 MHz (divider by 2 for USB)
  {BDI_INIT_ARM_DELAY, 0,	   100       }, //
  {BDI_INIT_ARM_WM32,  0xFFFFFC30, 0x00000202}, // MCKR : PCK/3 = MCK Master Clock = 59,904000MHz from PLLA
  {BDI_INIT_ARM_DELAY, 0,          100       }
};


/* Init list for MBX860 board */
#define CLOCK_MBX860    50000000
static const BDI_InitTypeT initListMBX860[] = {
  {BDI_INIT_PPC_WSPR, 638,        0xFA200000}, // IMMR  : IMMR: internal memory at 0xFA200000
  {BDI_INIT_PPC_WSPR, 158,        0x00000007}, // ICTRL : ICTRL: not serialized, no show cycles
  {BDI_INIT_PPC_WM32, 0xFA200000, 0x00612900}, // SIUMCR
  {BDI_INIT_PPC_WM32, 0xFA200004, 0xFFFFFF88}, // SYPCR : enable bus monitor, disable software watchdog
  {BDI_INIT_PPC_WM16, 0xFA200200, 0x00C2    }, // TBSCR : stop timebase and decrementer while freeze is asserted
  {BDI_INIT_PPC_WM32, 0xFA200320, 0x55CCAA33}, // RTCSCK: unlock real-time clock status and control register
  {BDI_INIT_PPC_WM16, 0xFA200220, 0x00C2    }, // RTCSC : stop real-time clock while freeze is asserted
  {BDI_INIT_PPC_WM16, 0xFA200240, 0x0082    }, // PISCR : stop periodic interrupt while freeze is asserted
  {BDI_INIT_PPC_WM32, 0xFA200280, 0x62000000}, // SCCR  : disable clock output
  {BDI_INIT_PPC_WM32, 0xFA200284, 0x0000D000}, // PLPRCR: clear flags
  {BDI_INIT_PPC_WM32, 0xFA200104, 0xFFE00940}, // OR0   : Flash 2MB, all accesses, CS early negate, 4ws
  {BDI_INIT_PPC_WM32, 0xFA20013C, 0xFF800940}, // OR7   : Flash 8bit, all accesses, CS early negate, 4ws
  {BDI_INIT_PPC_WM32, 0xFA200100, 0xFE000001}, // BR0   : Flash at 0xFE000000, 32bit, R/W, no parity, use GPCM
  {BDI_INIT_PPC_WM32, 0xFA200138, 0xFC000401}  // BR7   : Flash at 0xFC000000, 8bit, R/W, no parity, use GPCM
};


/* Initlist for ETAS MPC555 board */
#define CLOCK_MPC555    20000000
static BDI_InitTypeT initListMPC555[] = {
  {BDI_INIT_PPC_WMSR,   0,        0x00003002}, // MSR: set FP,ME,RI
  {BDI_INIT_PPC_WSPR,  27,        0x00003002}, // SRR1: set FP,ME,RI
  {BDI_INIT_PPC_WSPR, 638,        0x00000802}, // IMMR: InternalRegs to 0x00400000, Flash enabled
  {BDI_INIT_PPC_WSPR, 158,        0x00000007}, // ICTRL : ICTRL: not serialized, no show cycles
  {BDI_INIT_PPC_WM32, 0x006FC004, 0xFFFFFF01}, // SYPCR : disable watchdog
  {BDI_INIT_PPC_WM32, 0x006FC100, 0xFFF00003}, // BR0: TI Flash, single read, 32-bit, @ 0xFFF00000
  {BDI_INIT_PPC_WM32, 0x006FC104, 0xFFF00050}, // OR0: TI Flash, 5 wait, 1M block
  {BDI_INIT_PPC_WM32, 0x006FC108, 0x00000003}, // BR1: SRAM, single read, 32-bit, @0x00000000
  {BDI_INIT_PPC_WM32, 0x006FC10C, 0xFFF00000}  // OR1: SRAM 0 wait, 1M block
};


/* Init list for ADS860 board */
#define CLOCK_ADS860    20000000
static const BDI_InitTypeT initListADS860[] = {
  {BDI_INIT_PPC_WSPR, 638,        0x02200000}, // IMMR : internal memory at 0x02200000
  {BDI_INIT_PPC_WSPR, 158,        0x00000007}, // ICTRL:
  {BDI_INIT_PPC_WM32, 0x02200000, 0x01632440}, // SIUMCR
  {BDI_INIT_PPC_WM32, 0x02200004, 0xFFFFFF88}, // SYPCR
  {BDI_INIT_PPC_WM16, 0x02200200, 0x0002    }, // TBSCR
  {BDI_INIT_PPC_WM32, 0x02200320, 0x55CCAA33}, // RTCSCK: unlock real-time clock status and control register
  {BDI_INIT_PPC_WM16, 0x02200220, 0x0102    }, // RTCSC
  {BDI_INIT_PPC_WM16, 0x02200240, 0x0002    }  // PTSCR
};

/* Init list for 8349 board */
BDI_InitTypeT initList8349[] = {
  {BDI_INIT_PPC_WSPR, 8002,       0xFFF00100}, // Break at boot address 0xFFF00100
  {BDI_INIT_PPC_WSPR, 8009,       500       }, // Wait after Reset 0.5 seconds
  {BDI_INIT_PPC_WSPR, 8020,       0x8460A000}, // Reset Configuration Word High
  {BDI_INIT_PPC_WSPR, 8021,       0x04040000}, // Reset Configuration Word Low
};


/* Init list for 8360 board */
BDI_InitTypeT initList8360[] = {
  {BDI_INIT_PPC_WSPR, 8002,       0x00000100}, // Break at boot address 0x00000100
  {BDI_INIT_PPC_WSPR, 8009,       500       }, // Wait after Reset 0.5 seconds
  {BDI_INIT_PPC_WSPR, 8020,       0x80600000}, // Reset Configuration Word High
  {BDI_INIT_PPC_WSPR, 8021,       0x04040006}, // Reset Configuration Word Low
};



static void ProcessCommand(char* line)
{
  static char* nameTable[] = {
     "MD","MDH","MDB","MM","MMH","MMB","RD","RDI","RDCP","RM","RMI","RMCP",
     "STARTUP","INFO","JTAG","HELP","EXIT"
  };

  typedef enum {
    N_MD,N_MDH,N_MDB,N_MM,N_MMH,N_MMB,N_RD,N_RDI,N_RDCP,N_RM,N_RMI,N_RMCP,
    N_STARTUP,N_INFO,N_JTAG,N_HELP,N_EXIT
  } NameT;

  char*                 linePtr;
  int                   command;
  int                   paramCount;
  DWORD                 param1;
  DWORD                 param2;
  DWORD                 param3;
  int                   result;

  /* extract command and parameters */
  paramCount = 0;
  param1     = 0;
  param2     = 0;
  param3     = 0;

  /* get command */
  linePtr = MatchName(sizeof nameTable / sizeof nameTable[0],
                      nameTable,
                      &command,
                      line);

  /* get parameters */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param1, linePtr);
    paramCount++;
  } /* if */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param2, linePtr);
    paramCount++;
  } /* if */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param3, linePtr);
    paramCount++;
  } /* if */

  if ((command >= 0) && (ValidEndOfLine(linePtr))) {
    result = TARGET_OKAY;
    switch (command) {

      case N_MD:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayLong(param1, (WORD)param2);
        break;

      case N_MDH:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayWord(param1, (WORD)param2);
        break;

      case N_MDB:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayByte(param1, (WORD)param2);
        break;

      case N_MM:
        if (paramCount == 2)
          result = BDI_SetLong(param1, param2);
        else
          printf("# syntax is MM <address> <value>\n");
        break;

      case N_MMH:
        if (paramCount == 2)
          result = BDI_SetWord(param1, (WORD)param2);
        else
          printf("# syntax is MMH <address> <value>\n");
        break;

      case N_MMB:
        if (paramCount == 2)
          result = BDI_SetByte(param1, (BYTE)param2);
        else
          printf("# syntax is MMB <address> <value>\n");
        break;

      case N_RD:
        result = TELNET_DisplayGPR();
        break;

      case N_RDI:
        result = TELNET_DisplayICEBreaker();
        break;

      case N_RDCP:
        result = TELNET_DisplayCP15((WORD)param1);
        break;

      case N_RM:
        if (paramCount == 2)
          result = TARGET_WriteGPR((WORD)param1, param2);
        else
          printf("# syntax is RM <number> <value>\n");
        break;

      case N_RMI:
        if (paramCount == 2)
          result = TARGET_WriteICEBreaker((WORD)param1, param2);
        else
          printf("# syntax is RMI <number> <value>\n");
        break;

      case N_RMCP:
        if (paramCount == 2)
          result = TARGET_WriteCP15((WORD)param1, param2);
        else
          printf("# syntax is RMCP <number> <value>\n");
        break;

      case N_STARTUP:
        result = BDI_TargetStartup(100,
                                   12L << 8 | 1, /* MPC83xx, 8 MHz JTAG clock */
                                   sizeof initList8360 / sizeof initList8360[0],
                                   initList8360);
        if (result == TARGET_OKAY) result = DisplayTargetInfo();
        break;

      case N_INFO:
        result = DisplayTargetInfo();
        break;

      case N_JTAG:
        result = BDI_DoJtag(JTAG_CMD_OPEN, 0, 0, NULL, NULL);
        if (result == TARGET_OKAY) modeJTAG = TRUE;
        break;

      case N_HELP:
        DisplayHelp();
        break;

      case N_EXIT:
        terminate = TRUE;
        break;

      default:
        printf("Sorry, command not jet implemented!\n");
        break;

    } /* switch */

    /* display error */
    if (result != TARGET_OKAY) {
      printf("%s\n", BDI_ErrorTextGet(result));
    } /* if */
  } /* if */

  else {
    printf("# syntax error in command\n");
  } /* else */

} /* ProcessCommand */




/****************************************************************************
 ****************************************************************************

    TELNET_ProcessJTAG :

    Process a JTAG command line

    INPUT  : line       the command line
    OUTPUT : -

 ****************************************************************************/

#define JTAG_DATA_BYTES         32
#define JTAG_DATA_BITS          (8 * JTAG_DATA_BYTES)

/* Binary to ASCII-HEX conversion table */
static const char BinToHex[] = {'0','1','2','3','4','5','6','7','8','9',
                                'a','b','c','d','e','f'};

static char* jtagHelp[] = {
"TRST  {0|1}                  assert (1) or release (0) TRST",
"RST   {0|1}                  assert (1) or release (0) RST",
"HALT  {0|1}                  assert (1) or release (0) DINT",
"SPEED <n>                    select JTAG clock frequency",
"CLK   <count> <tms>          clock TAP with requested TMS value",
"SCAN  <ctrl><len><...b2b1b0> scan with provided parameters",
"RIR   <len>                  read  IR, zero is scanned in",
"RDR   <len>                  read  DR, zero is scanned in",
"WIR   <len> <...b2b1b0>      write IR, b0 is first scanned",
"WDR   <len> <...b2b1b0>      write DR, b0 is first scanned",
"XIR   <len> <...b2b1b0>      xchg  IR, b0 is first scanned",
"XDR   <len> <...b2b1b0>      xchg  DR, b0 is first scanned",
"                             len : the number of bits 1..256",
"                             bx  : a data byte, two hex digits",
"HELP                         display JTAG command list",
"EXIT                         terminate JTAG mode"
};

static void DisplayJtagHelp(void)
{
  int   i;
  char  line[80];

  for (i=0; i < (int)(sizeof jtagHelp / sizeof jtagHelp[0]); i++) {
    printf("%s\n", jtagHelp[i]);
  } /* for */
} /* DisplayJtagHelp */


static const char* ExtractHexByte(BYTE* value, const char* getPtr)
{
  BYTE  num;
  BYTE  digit;
  BYTE  c;
  int   i;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (i = 0; i < 2; i++) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (BYTE)(c - '0');
    else if ((c >= 'a') && (c <= 'f')) digit = (BYTE)(c - 'a' + 10);
    else if ((c >= 'A') && (c <= 'F')) digit = (BYTE)(c - 'A' + 10);
    else break;
    num <<= 4;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractHexByte */


static void ProcessJTAG(char* line)
{
  static char* jtagTable[] = {
     "TRST","RST","HALT","CLK","SPEED","SCAN",
     "RIR","RDR","WIR","WDR","XIR","XDR","HELP","EXIT"
  };

  typedef enum {
    N_TRST,N_RST,N_HALT,N_CLK,N_SPEED,N_SCAN,
    N_RIR,N_RDR,N_WIR,N_WDR,N_XIR,N_XDR,N_HELP,N_EXIT
  } JtagT;

  const char*   linePtr;
  int           result;
  int           command;
  int           paramCount;
  DWORD         param1;
  DWORD         param2;
  DWORD         index;
  BYTE          dataWR[JTAG_DATA_BYTES];
  BYTE          dataRD[JTAG_DATA_BYTES];
  char          szData[2 * JTAG_DATA_BYTES + 1];
  char*         putPtr;

  /* extract command and parameters */
  paramCount = 0;
  param1     = 0;
  param2     = 0;
  (void)memset(dataRD, 0, sizeof dataRD);
  (void)memset(dataWR, 0, sizeof dataWR);
  linePtr = MatchName(sizeof jtagTable / sizeof jtagTable[0],
                      jtagTable,
                      &command,
                      line);

  /* get 1. parameter */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param1, (char*)linePtr);
    paramCount++;
    if (   (command == N_WIR) || (command == N_WDR)
        || (command == N_XIR) || (command == N_XDR)) {
      if (param1 > JTAG_DATA_BITS) {
        printf("# maximum data length is 256 bits\n");
        return;
      } /* if */
      if (param1 == 0) {
        printf("# minimal data length is 1 bit\n");
        return;
      } /* if */
    } /* if */
  } /* if */

  /* get 2. parameter */
  if (!ValidEndOfLine(linePtr)) {
    if ((command == N_WIR) || (command == N_WDR) || (command == N_XIR) || (command == N_XDR)) {
      while (*linePtr == ' ') linePtr++;
      index = (param1 + 7) / 8;
      if (strlen(linePtr) != (2 * index)) {
        printf("# data entry does not match length\n");
        return;
      } /* if */
      while (index > 0) {
        index--;
        linePtr = ExtractHexByte(&dataWR[index], linePtr);
      } /* while */
    } /* if */
    else {
      linePtr = ExtractNumber(&param2, (char*)linePtr);
    } /* else */
    paramCount++;
    if (command == N_SCAN) {
      if (param2 > JTAG_DATA_BITS) {
        printf("# maximum data length is 256 bits\n");
        return;
      } /* if */
      if (param2 == 0) {
        printf("# minimal data length is 1 bit\n");
        return;
      } /* if */
    } /* if */
  } /* if */

  /* get 3. parameter */
  if (!ValidEndOfLine(linePtr)) {
    if (command == N_SCAN) {
      while (*linePtr == ' ') linePtr++;
      index = (param2 + 7) / 8;
      if (strlen(linePtr) != (2 * index)) {
        printf("# data entry does not match length\n");
        return;
      } /* if */
      while (index > 0) {
        index--;
        linePtr = ExtractHexByte(&dataWR[index], linePtr);
      } /* while */
    } /* if */
    paramCount++;
  } /* if */

  /* process command */
  result = TARGET_OKAY;
  if ((command >= 0) && (ValidEndOfLine(linePtr))) {
    switch (command) {

      case N_TRST:
        result = BDI_DoJtag(JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 0),
                            0,
                            NULL,
                            NULL);
        break;

      case N_RST:
        result = BDI_DoJtag(JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 2),
                            0,
                            NULL,
                            NULL);
        break;

      case N_HALT:
        result = BDI_DoJtag(JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 4),
                            0,
                            NULL,
                            NULL);
        break;

      case N_CLK:
        if ((param1 >= 1) && (param1 <= 0xFFFF)) {
          result = BDI_DoJtag(JTAG_CMD_CLOCK,
                              (BYTE)((param2) ? JTAG_CLOCK_TMS : 0),
                              (WORD)param1,
                              NULL,
                              NULL);
        } /* if */
        else {
          printf("# valid count is 1...64K\n");
        } /* else */
        break;

      case N_SPEED:
        if (param1 <= 6) {
          result = BDI_DoJtag(JTAG_CMD_SPEED,
                              (BYTE)param1,
                              0,
                              NULL,
                              NULL);
        } /* if */
        else {
          printf("# valid speed is 0...6\n");
        } /* else */
        break;

      case N_SCAN:
        result = BDI_DoJtag(JTAG_CMD_SCAN,
                            (BYTE)param1,
                            (WORD)param2,
                            dataWR,
                            dataRD);
        break;

      case N_RIR:
      case N_RDR:
        result = BDI_DoJtag(JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag((BYTE)((command == N_RIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            dataRD);
        break;

      case N_WIR:
      case N_WDR:
        result = BDI_DoJtag(JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag((BYTE)((command == N_WIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            NULL);
        break;

      case N_XIR:
      case N_XDR:
        result = BDI_DoJtag(JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag((BYTE)((command == N_XIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            dataRD);
        break;

      case N_HELP:
        DisplayJtagHelp();
        break;

      case N_EXIT:
        result = BDI_DoJtag(JTAG_CMD_CLOSE, 0, 0, NULL, NULL);
        modeJTAG = FALSE;
        break;

      default:
        printf("Sorry, command not jet implemented!\n");
        break;

    } /* switch */

    /* display error */
    if (result != TARGET_OKAY) {
      printf("%s\n", BDI_ErrorTextGet(result));
    } /* if */

    /* display data */
    if (    (command == N_RIR) || (command == N_RDR)
         || (command == N_XIR) || (command == N_XDR)
         || (command == N_SCAN)) {
      index = (command == N_SCAN) ? param2 : param1;
      index = (index + 7) / 8;
      putPtr = szData;
      while (index > 0) {
        index--;
        *putPtr++ = BinToHex[(dataRD[index] >> 4) & 0x0F];
        *putPtr++ = BinToHex[dataRD[index] & 0x0F];
      } /* while */
      *putPtr = 0;
      printf("%s\n", szData);
    } /* if */

  } /* if */
  else {
    printf("# invalid command in JTAG mode\n");
    printf("# enter exit to terminate JTAG mode\n");
  } /* else */

} /* ProcessJTAG */


/*************************************************************************
|  MAIN
|*************************************************************************/

int main( int argc, char *argv[ ])
{
  char  rxLine[128];
  char  host[80];
  int   param;
  int   result;
  WORD  bdiType;
  WORD  fwType;
  WORD  fwVersion;

  /* connect to BDI */
  printf("Connecting to BDI, please wait...\r");
  if (argc == 3) {
    sprintf(host, "%s", argv[1]);
    param = atoi(argv[2]);
  } /* if */
  else  {
    sprintf(host, "151.120.25.101");
    param = 2001;
  } /* if */
  result = BDI_Connect(host, param);
  if (result != 0) {
    printf("Connecting to BDI failed with error %i\n", result);
    printf("press enter to continue...");
    gets(rxLine);
    exit(0);
  } /* if */
  result = BDI_TypeGet(&bdiType, &fwType, &fwVersion);
  printf("\n\n");
  printf("BDI: %i FW: %i / %i \n", bdiType, fwType, fwVersion);

  printf("\n\n");
  printf("Simple bdiAccess Demo\n");
  printf("=====================\n\n");
  printf("enter HELP for command list\n");

  terminate = FALSE;
  modeJTAG  = FALSE;
  do {
    printf((modeJTAG) ? "JTAG>" : "BDI>");
    if (gets(rxLine)) {
      if (modeJTAG) ProcessJTAG(rxLine);
      else          ProcessCommand(rxLine);
    } /* if */
  } while (!terminate);

  result = BDI_Disconnect();

  return 0;
} /* main */
